From s0mbre@uganda.factory.vocord.ru Wed Jul 27 02:12:30 2005
Date: Wed, 27 Jul 2005 13:10:11 +0400
From: Evgeniy Polyakov <johnpol@2ka.mipt.ru>
To: GregKH <greg@kroah.com>
Subject: W1: w1_netlink: New init/fini netlink callbacks.
Message-ID: <20050727091011.GA19677@tservice.net.ru>

They are guarded with NETLINK_DISABLE compile time options,
so if CONFIG_NET is disabled, no linking errors occur.
Bug noticed by Adrian Bunk <bunk@stusta.de>.

Signed-off-by: Evgeniy Polyakov <johnpol@2ka.mipt.ru>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>

---
 drivers/w1/w1_int.c     |   16 ++++++----------
 drivers/w1/w1_netlink.c |   26 ++++++++++++++++++++++++++
 drivers/w1/w1_netlink.h |    2 ++
 3 files changed, 34 insertions(+), 10 deletions(-)

--- gregkh-2.6.orig/drivers/w1/w1_int.c	2005-07-27 13:02:51.000000000 -0700
+++ gregkh-2.6/drivers/w1/w1_int.c	2005-07-27 13:29:47.000000000 -0700
@@ -88,17 +88,14 @@
 
 	dev->groups = 23;
 	dev->seq = 1;
-	dev->nls = netlink_kernel_create(NETLINK_W1, NULL);
-	if (!dev->nls) {
-		printk(KERN_ERR "Failed to create new netlink socket(%u) for w1 master %s.\n",
-			NETLINK_NFLOG, dev->dev.bus_id);
-	}
+	dev_init_netlink(dev);
 
 	err = device_register(&dev->dev);
 	if (err) {
 		printk(KERN_ERR "Failed to register master device. err=%d\n", err);
-		if (dev->nls && dev->nls->sk_socket)
-			sock_release(dev->nls->sk_socket);
+
+		dev_fini_netlink(dev);
+
 		memset(dev, 0, sizeof(struct w1_master));
 		kfree(dev);
 		dev = NULL;
@@ -107,11 +104,10 @@
 	return dev;
 }
 
-static void w1_free_dev(struct w1_master *dev)
+void w1_free_dev(struct w1_master *dev)
 {
 	device_unregister(&dev->dev);
-	if (dev->nls && dev->nls->sk_socket)
-		sock_release(dev->nls->sk_socket);
+	dev_fini_netlink(dev);
 	memset(dev, 0, sizeof(struct w1_master) + sizeof(struct w1_bus_master));
 	kfree(dev);
 }
--- gregkh-2.6.orig/drivers/w1/w1_netlink.c	2005-07-27 12:53:08.000000000 -0700
+++ gregkh-2.6/drivers/w1/w1_netlink.c	2005-07-27 13:28:57.000000000 -0700
@@ -57,10 +57,36 @@
 nlmsg_failure:
 	return;
 }
+
+int dev_init_netlink(struct w1_master *dev)
+{
+	dev->nls = netlink_kernel_create(NETLINK_W1, NULL);
+	if (!dev->nls) {
+		printk(KERN_ERR "Failed to create new netlink socket(%u) for w1 master %s.\n",
+			NETLINK_W1, dev->dev.bus_id);
+	}
+
+	return 0;
+}
+
+void dev_fini_netlink(struct w1_master *dev)
+{
+	if (dev->nls && dev->nls->sk_socket)
+		sock_release(dev->nls->sk_socket);
+}
 #else
 #warning Netlink support is disabled. Please compile with NET support enabled.
 
 void w1_netlink_send(struct w1_master *dev, struct w1_netlink_msg *msg)
 {
 }
+
+int dev_init_netlink(struct w1_master *dev)
+{
+	return 0;
+}
+
+void dev_fini_netlink(struct w1_master *dev)
+{
+}
 #endif
--- gregkh-2.6.orig/drivers/w1/w1_netlink.h	2005-07-27 12:53:08.000000000 -0700
+++ gregkh-2.6/drivers/w1/w1_netlink.h	2005-07-27 13:28:57.000000000 -0700
@@ -52,6 +52,8 @@
 #ifdef __KERNEL__
 
 void w1_netlink_send(struct w1_master *, struct w1_netlink_msg *);
+int dev_init_netlink(struct w1_master *dev);
+void dev_fini_netlink(struct w1_master *dev);
 
 #endif /* __KERNEL__ */
 #endif /* __W1_NETLINK_H */
